@@ -75,7 +75,7 @@ module ActiveRecord
       end
 
       def quoted_date(value)
-        if value.acts_like?(:time) && value.respond_to?(:usec)
+        if supports_datetime_with_precision? && value.acts_like?(:time) && value.respond_to?(:usec)
           "#{super}.#{sprintf("%06d", value.usec)}"
         else
           super
