@@ -1,14 +1,14 @@
 module ActiveSupport
-  # Returns the version of the currently loaded ActiveSupport as a <tt>Gem::Version</tt>
+  # Returns the version of the currently loaded Active Support as a <tt>Gem::Version</tt>
   def self.gem_version
     Gem::Version.new VERSION::STRING
   end
 
   module VERSION
     MAJOR = 4
-    MINOR = 1
-    TINY  = 10
-    PRE   = nil
+    MINOR = 2
+    TINY  = 7
+    PRE   = "1"
 
     STRING = [MAJOR, MINOR, TINY, PRE].compact.join(".")
   end
