@@ -13,26 +13,11 @@ class HelperGeneratorTest < Rails::Generators::TestCase
     assert_file "app/helpers/admin_helper.rb", /module AdminHelper/
   end
 
-  def test_invokes_default_test_framework
-    run_generator
-    assert_file "test/unit/helpers/admin_helper_test.rb", /class AdminHelperTest < ActionView::TestCase/
-  end
-
-  def test_logs_if_the_test_framework_cannot_be_found
-    content = run_generator ["admin", "--test-framework=rspec"]
-    assert_match(/rspec \[not found\]/, content)
-  end
-
   def test_check_class_collision
     content = capture(:stderr){ run_generator ["object"] }
     assert_match(/The name 'ObjectHelper' is either already used in your application or reserved/, content)
   end
 
-  def test_check_class_collision_on_tests
-    content = capture(:stderr){ run_generator ["another_object"] }
-    assert_match(/The name 'AnotherObjectHelperTest' is either already used in your application or reserved/, content)
-  end
-
   def test_namespaced_and_not_namespaced_helpers
     run_generator ["products"]
 
