@@ -135,7 +135,7 @@ namespace :all do
   end
 
   task :tag do
-    sh "git tag -m '#{tag} release' #{tag}"
+    sh "git tag -s -m '#{tag} release' #{tag}"
     sh "git push --tags"
   end
 
